@@ -36,7 +36,7 @@ module Sequel
       # type doesn't match a known type, just return the value.
       def result_value_of(declared_type, value)
         if value.is_a?(::Amalgalite::Blob)
-          SQL::Blob.new(value.source)
+          SQL::Blob.new(value.to_s)
         elsif value.is_a?(String) && declared_type
           (meth = self.class.sql_to_method(declared_type.downcase)) ? send(meth, value) : value
         else
@@ -84,37 +84,42 @@ module Sequel
       
       # Run the given SQL with the given arguments. Returns nil.
       def execute_ddl(sql, opts={})
-        _execute(sql, opts){|conn| conn.execute_batch(sql);}
+        _execute(sql, opts){|conn| log_yield(sql){conn.execute_batch(sql)}}
         nil
       end
       
       # Run the given SQL with the given arguments and return the number of changed rows.
       def execute_dui(sql, opts={})
-        _execute(sql, opts){|conn| conn.execute_batch(sql); conn.row_changes}
+        _execute(sql, opts){|conn| log_yield(sql){conn.execute_batch(sql)}; conn.row_changes}
       end
       
       # Run the given SQL with the given arguments and return the last inserted row id.
       def execute_insert(sql, opts={})
-        _execute(sql, opts){|conn| conn.execute_batch(sql); conn.last_insert_rowid}
+        _execute(sql, opts){|conn| log_yield(sql){conn.execute_batch(sql)}; conn.last_insert_rowid}
       end
       
       # Run the given SQL with the given arguments and yield each row.
-      def execute(sql, opts={}, &block)
-        _execute(sql, opts){|conn| conn.prepare(sql, &block)}
+      def execute(sql, opts={})
+        _execute(sql, opts) do |conn|
+          begin
+            yield(stmt = log_yield(sql){conn.prepare(sql)})
+          ensure
+            stmt.close if stmt
+          end
+        end
       end
       
       # Run the given SQL with the given arguments and return the first value of the first row.
       def single_value(sql, opts={})
-        _execute(sql, opts){|conn| conn.first_value_from(sql)}
+        _execute(sql, opts){|conn| log_yield(sql){conn.first_value_from(sql)}}
       end
       
       private
       
-      # Log the SQL and yield an available connection.  Rescue
+      # Yield an available connection.  Rescue
       # any Amalgalite::Errors and turn them into DatabaseErrors.
       def _execute(sql, opts)
         begin
-          log_info(sql)
           synchronize(opts[:server]){|conn| yield conn}
         rescue ::Amalgalite::Error, ::Amalgalite::SQLite3::Error => e
           raise_error(e)
@@ -125,7 +130,7 @@ module Sequel
       # Also, force the max connections to 1 if a memory database is being
       # used, as otherwise each connection gets a separate database.
       def connection_pool_default_options
-        o = super.merge(:pool_convert_exceptions=>false)
+        o = super.dup
         # Default to only a single connection if a memory database is used,
         # because otherwise each connection will get a separate database
         o[:max_connections] = 1 if @opts[:database] == ':memory:' || blank_object?(@opts[:database])
