@@ -43,15 +43,13 @@ module Sequel
       end
 
       def execute(sql, opts={})
-        log_info(sql)
         begin
           synchronize(opts[:server]) do |conn|
-            r = conn.execute(sql)
+            r = log_yield(sql){conn.execute(sql)}
             yield(r) if block_given?
             r
           end
         rescue => e
-          log_info(e.message)
           raise_error(e, :classes=>[Fb::Error])
         end
       end
@@ -104,15 +102,13 @@ module Sequel
         AUTO_INCREMENT
       end
       
-      def begin_transaction(conn)
-        log_info(TRANSACTION_BEGIN)
-        conn.transaction
+      def begin_transaction(conn, opts={})
+        log_yield(TRANSACTION_BEGIN){conn.transaction}
         conn
       end
 
-      def commit_transaction(conn)
-        log_info(TRANSACTION_COMMIT)
-        conn.commit
+      def commit_transaction(conn, opts={})
+        log_yield(TRANSACTION_COMMIT){conn.commit}
       end
       
       def create_sequence_sql(name, opts={})
@@ -160,7 +156,7 @@ module Sequel
         events = opts[:events] ? Array(opts[:events]) : [:insert, :update, :delete]
         whence = opts[:after] ? 'AFTER' : 'BEFORE'
         inactive = opts[:inactive] ? 'INACTIVE' : 'ACTIVE'
-        position = opts[:position] ? opts[:position] : 0
+        position = opts.fetch(:position, 0)
         sql = <<-end_sql
           CREATE TRIGGER #{quote_identifier(name)} for #{quote_identifier(table)}
           #{inactive} #{whence} #{events.map{|e| e.to_s.upcase}.join(' OR ')} position #{position}
@@ -186,9 +182,8 @@ module Sequel
         "ALTER SEQUENCE #{seq_name} RESTART WITH #{opts[:restart_position]}"
       end
       
-      def rollback_transaction(conn)
-        log_info(TRANSACTION_ROLLBACK)
-        conn.rollback
+      def rollback_transaction(conn, opts={})
+        log_yield(TRANSACTION_ROLLBACK){conn.rollback}
       end
 
       def type_literal_generic_string(column)
@@ -206,7 +201,7 @@ module Sequel
 
       # Yield all rows returned by executing the given SQL and converting
       # the types.
-      def fetch_rows(sql, &block)
+      def fetch_rows(sql)
         execute(sql) do |s|
           begin
             @columns = s.fields.map{|c| output_identifier(c.name)}
