@@ -3,24 +3,6 @@ Sequel.require 'adapters/utils/stored_procedures'
 
 module Sequel
   # Houses Sequel's JDBC support when running on JRuby.
-  # Support for individual database types is done using sub adapters.
-  # PostgreSQL, MySQL, SQLite, Oracle, and MSSQL all have relatively good support,
-  # close the the level supported by the native adapter.
-  # PostgreSQL, MySQL, SQLite can load necessary support using
-  # the jdbc-* gem, if it is installed, though they will work if you
-  # have the correct .jar in your CLASSPATH.  Oracle and MSSQL should
-  # load the necessary support if you have the .jar in your CLASSPATH.
-  # For all other databases, the Java class should be loaded manually
-  # before calling Sequel.connect.
-  #
-  # Note that when using a JDBC adapter, the best way to use Sequel
-  # is via Sequel.connect, NOT Sequel.jdbc.  Use the JDBC connection
-  # string when connecting, which will be in a different format than
-  # the native connection string.  The connection string should start
-  # with 'jdbc:'.  For PostgreSQL, use 'jdbc:postgresql:', and for
-  # SQLite you do not need 2 preceding slashes for the database name
-  # (use no preceding slashes for a relative path, and one preceding
-  # slash for an absolute path).
   module JDBC
     # Make it accesing the java.lang hierarchy more ruby friendly.
     module JavaLang
@@ -31,42 +13,66 @@ module Sequel
     module JavaSQL
       include_package 'java.sql'
     end
+
+    # Make it accesing the javax.naming hierarchy more ruby friendly.
+    module JavaxNaming
+      include_package 'javax.naming'
+    end
+
+    # Used to identify a jndi connection and to extract the jndi
+    # resource name.
+    JNDI_URI_REGEXP = /\Ajdbc:jndi:(.+)/
+    
+    # The types to check for 0 scale to transform :decimal types
+    # to :integer.
+    DECIMAL_TYPE_RE = /number|numeric|decimal/io
     
     # Contains procs keyed on sub adapter type that extend the
     # given database object so it supports the correct database type.
     DATABASE_SETUP = {:postgresql=>proc do |db|
-        Sequel.require 'adapters/jdbc/postgresql'
+        Sequel.ts_require 'adapters/jdbc/postgresql'
         db.extend(Sequel::JDBC::Postgres::DatabaseMethods)
         JDBC.load_gem('postgres')
         org.postgresql.Driver
       end,
       :mysql=>proc do |db|
-        Sequel.require 'adapters/jdbc/mysql'
+        Sequel.ts_require 'adapters/jdbc/mysql'
         db.extend(Sequel::JDBC::MySQL::DatabaseMethods)
         JDBC.load_gem('mysql')
         com.mysql.jdbc.Driver
       end,
       :sqlite=>proc do |db|
-        Sequel.require 'adapters/jdbc/sqlite'
+        Sequel.ts_require 'adapters/jdbc/sqlite'
         db.extend(Sequel::JDBC::SQLite::DatabaseMethods)
         JDBC.load_gem('sqlite3')
         org.sqlite.JDBC
       end,
       :oracle=>proc do |db|
-        Sequel.require 'adapters/jdbc/oracle'
+        Sequel.ts_require 'adapters/jdbc/oracle'
         db.extend(Sequel::JDBC::Oracle::DatabaseMethods)
         Java::oracle.jdbc.driver.OracleDriver
       end,
       :sqlserver=>proc do |db|
-        Sequel.require 'adapters/jdbc/mssql'
+        Sequel.ts_require 'adapters/jdbc/mssql'
         db.extend(Sequel::JDBC::MSSQL::DatabaseMethods)
         com.microsoft.sqlserver.jdbc.SQLServerDriver
       end,
+      :jtds=>proc do |db|
+        Sequel.ts_require 'adapters/jdbc/mssql'
+        db.extend(Sequel::JDBC::MSSQL::DatabaseMethods)
+        JDBC.load_gem('jtds')
+        Java::net.sourceforge.jtds.jdbc.Driver
+      end,
       :h2=>proc do |db|
-        Sequel.require 'adapters/jdbc/h2'
+        Sequel.ts_require 'adapters/jdbc/h2'
         db.extend(Sequel::JDBC::H2::DatabaseMethods)
         JDBC.load_gem('h2')
         org.h2.Driver
+      end,
+      :as400=>proc do |db|
+        Sequel.ts_require 'adapters/jdbc/as400'
+        db.extend(Sequel::JDBC::AS400::DatabaseMethods)
+        com.ibm.as400.access.AS400JDBCDriver
       end
     }
     
@@ -74,7 +80,7 @@ module Sequel
     # works for PostgreSQL, MySQL, and SQLite.
     def self.load_gem(name)
       begin
-        require "jdbc/#{name}"
+        Sequel.tsk_require "jdbc/#{name}"
       rescue LoadError
         # jdbc gem not used, hopefully the user has the .jar in their CLASSPATH
       end
@@ -88,6 +94,9 @@ module Sequel
       # The type of database we are connecting to
       attr_reader :database_type
       
+      # The Java database driver we are using
+      attr_reader :driver
+      
       # Whether to convert some Java types to ruby types when retrieving rows.
       # True by default, can be set to false to roughly double performance when
       # fetching rows.
@@ -98,13 +107,15 @@ module Sequel
       # raise an error immediately if the connection doesn't have a
       # uri, since JDBC requires one.
       def initialize(opts)
-        @opts = opts
-        @convert_types = opts.include?(:convert_types) ? typecast_value_boolean(opts[:convert_types]) : true
+        super
+        @convert_types = typecast_value_boolean(@opts.fetch(:convert_types, true))
         raise(Error, "No connection string specified") unless uri
-        if match = /\Ajdbc:([^:]+)/.match(uri) and prok = DATABASE_SETUP[match[1].to_sym]
-          prok.call(self)
-        end
-        super(opts)
+        
+        resolved_uri = jndi? ? get_uri_from_jndi : uri
+
+        if match = /\Ajdbc:([^:]+)/.match(resolved_uri) and prok = DATABASE_SETUP[match[1].to_sym]
+          @driver = prok.call(self)
+        end        
       end
       
       # Execute the given stored procedure with the give name. If a block is
@@ -120,14 +131,14 @@ module Sequel
 
           begin
             if block_given?
-              yield cps.executeQuery
+              yield log_yield(sql){cps.executeQuery}
             else
               case opts[:type]
               when :insert
-                cps.executeUpdate
+                log_yield(sql){cps.executeUpdate}
                 last_insert_id(conn, opts)
               else
-                cps.executeUpdate
+                log_yield(sql){cps.executeUpdate}
               end
             end
           rescue NativeException, JavaSQL::SQLException => e
@@ -137,12 +148,35 @@ module Sequel
           end
         end
       end
-      
+         
       # Connect to the database using JavaSQL::DriverManager.getConnection.
       def connect(server)
-        args = [uri(server_opts(server))]
-        args.concat([opts[:user], opts[:password]]) if opts[:user] && opts[:password]
-        setup_connection(JavaSQL::DriverManager.getConnection(*args))
+        opts = server_opts(server)
+        conn = if jndi?
+          get_connection_from_jndi
+        else
+          args = [uri(opts)]
+          args.concat([opts[:user], opts[:password]]) if opts[:user] && opts[:password]
+          begin
+            JavaSQL::DriverManager.getConnection(*args)
+          rescue => e
+            raise e unless driver
+            # If the DriverManager can't get the connection - use the connect
+            # method of the driver. (This happens under Tomcat for instance)
+            props = java.util.Properties.new
+            if opts && opts[:user] && opts[:password]
+              props.setProperty("user", opts[:user])
+              props.setProperty("password", opts[:password])
+            end
+            begin
+              driver.new.connect(args[0], props)
+            rescue => e2
+              e.message << "\n#{e2.class.name}: #{e2.message}"
+              raise e
+            end
+          end
+        end
+        setup_connection(conn)
       end
       
       # Return instances of JDBC::Dataset with the given opts.
@@ -155,27 +189,27 @@ module Sequel
       def execute(sql, opts={}, &block)
         return call_sproc(sql, opts, &block) if opts[:sproc]
         return execute_prepared_statement(sql, opts, &block) if [Symbol, Dataset].any?{|c| sql.is_a?(c)}
-        log_info(sql)
         synchronize(opts[:server]) do |conn|
-          stmt = conn.createStatement
-          begin
-            if block_given?
-              yield stmt.executeQuery(sql)
+          statement(conn) do |stmt|
+            if block
+              yield log_yield(sql){stmt.executeQuery(sql)}
             else
               case opts[:type]
               when :ddl
-                stmt.execute(sql)
+                log_yield(sql){stmt.execute(sql)}
               when :insert
-                stmt.executeUpdate(sql)
+                log_yield(sql) do
+                  if requires_return_generated_keys?
+                    stmt.executeUpdate(sql, JavaSQL::Statement.RETURN_GENERATED_KEYS)
+                  else
+                    stmt.executeUpdate(sql)
+                  end
+                end
                 last_insert_id(conn, opts.merge(:stmt=>stmt))
               else
-                stmt.executeUpdate(sql)
+                log_yield(sql){stmt.executeUpdate(sql)}
               end
             end
-          rescue NativeException, JavaSQL::SQLException => e
-            raise_error(e)
-          ensure
-            stmt.close
           end
         end
       end
@@ -193,10 +227,7 @@ module Sequel
         execute(sql, {:type=>:insert}.merge(opts))
       end
       
-      # Return a hash containing index information. Hash keys are index name symbols.
-      # Values are subhashes with two keys, :columns and :unique.  The value of :columns
-      # is an array of symbols of column names.  The value of :unique is true or false
-      # depending on if the index is unique.
+      # Use the JDBC metadata to get the index information for the table.
       def indexes(table, opts={})
         m = output_identifier_meth
         im = input_identifier_meth
@@ -215,7 +246,7 @@ module Sequel
       end 
 
       # All tables in this database
-      def tables
+      def tables(opts={})
         ts = []
         m = output_identifier_meth
         metadata(:getTables, nil, nil, nil, ['TABLE'].to_java(:string)){|h| ts << m.call(h[:table_name])}
@@ -231,20 +262,14 @@ module Sequel
         ur = opts[:uri] || opts[:url] || opts[:database]
         ur =~ /^\Ajdbc:/ ? ur : "jdbc:#{ur}"
       end
+
+      # Whether or not JNDI is being used for this connection.
+      def jndi?
+        !!(uri =~ JNDI_URI_REGEXP)
+      end
       
       private
-      
-      # JDBC uses a statement object to execute SQL on the database
-      def begin_transaction(conn)
-        conn = conn.createStatement unless supports_savepoints?
-        super
-      end
-      
-      # The JDBC adapter should not need the pool to convert exceptions.
-      def connection_pool_default_options
-        super.merge(:pool_convert_exceptions=>false)
-      end
-      
+         
       # Close given adapter connections
       def disconnect_connection(c)
         c.close
@@ -270,29 +295,25 @@ module Sequel
           if name and cps = conn.prepared_statements[name] and cps[0] == sql
             cps = cps[1]
           else
-            if cps
-              log_info("Closing #{name}")
-              cps[1].close
-            end
-            log_info("Preparing#{" #{name}:" if name} #{sql}")
-            cps = conn.prepareStatement(sql)
+            log_yield("Closing #{name}"){cps[1].close} if cps
+            cps = log_yield("Preparing#{" #{name}:" if name} #{sql}"){conn.prepareStatement(sql)}
             conn.prepared_statements[name] = [sql, cps] if name
           end
           i = 0
           args.each{|arg| set_ps_arg(cps, arg, i+=1)}
-          log_info("Executing#{" #{name}" if name}", args)
+          msg = "Executing#{" #{name}" if name}"
           begin
             if block_given?
-              yield cps.executeQuery
+              yield log_yield(msg, args){cps.executeQuery}
             else
               case opts[:type]
               when :ddl
-                cps.execute
+                log_yield(msg, args){cps.execute}
               when :insert
-                cps.executeUpdate
+                log_yield(msg, args){cps.executeUpdate}
                 last_insert_id(conn, opts.merge(:prepared=>true))
               else
-                cps.executeUpdate
+                log_yield(msg, args){cps.executeUpdate}
               end
             end
           rescue NativeException, JavaSQL::SQLException => e
@@ -302,13 +323,44 @@ module Sequel
           end
         end
       end
+
+      # Gets the JDBC connection uri from the JNDI resource.
+      def get_uri_from_jndi
+        conn = get_connection_from_jndi
+        conn.meta_data.url
+      ensure
+        conn.close if conn
+      end
       
+      # Gets the connection from JNDI.
+      def get_connection_from_jndi
+        jndi_name = JNDI_URI_REGEXP.match(uri)[1]
+        JavaxNaming::InitialContext.new.lookup(jndi_name).connection
+      end
+            
+      # Support Date objects used in bound variables
+      def java_sql_date(date)
+        java.sql.Date.new(Time.local(date.year, date.month, date.day).to_i * 1000)
+      end
+
+      # Support DateTime objects used in bound variables
+      def java_sql_datetime(datetime)
+        ts = java.sql.Timestamp.new(Time.local(datetime.year, datetime.month, datetime.day, datetime.hour, datetime.min, datetime.sec).to_i * 1000)
+        ts.setNanos((datetime.sec_fraction * (RUBY_VERSION >= '1.9.0' ?  1000000000 : 86400000000000)).to_i)
+        ts
+      end
+
       # Support fractional seconds for Time objects used in bound variables
       def java_sql_timestamp(time)
-        millis = time.to_i * 1000
-        ts = java.sql.Timestamp.new(millis)
-        ts.setNanos(time.usec * 1000)
+        ts = java.sql.Timestamp.new(time.to_i * 1000)
+        ts.setNanos(RUBY_VERSION >= '1.9.0' ? time.nsec : time.usec * 1000)
         ts
+      end 
+      
+      # Log the given SQL and then execute it on the connection, used by
+      # the transaction code.
+      def log_connection_execute(conn, sql)
+        statement(conn){|s| log_yield(sql){s.execute(sql)}}
       end
 
       # By default, there is no support for determining the last inserted
@@ -320,15 +372,22 @@ module Sequel
       
       # Yield the metadata for this database
       def metadata(*args, &block)
-        synchronize{|c| metadata_dataset.send(:process_result_set, c.getMetaData.send(*args), &block)}
+        synchronize do |c|
+          result = c.getMetaData.send(*args)
+          begin
+            metadata_dataset.send(:process_result_set, result, &block)
+          ensure
+            result.close
+          end
+        end
       end
-      
-      # Close the given statement when removing the transaction
-      def remove_transaction(stmt)
-        stmt.close if stmt && !supports_savepoints?
-        super
+
+      # Treat SQLExceptions with a "Connection Error" SQLState as disconnects
+      def raise_error(exception, opts={})
+        cause = exception.respond_to?(:cause) ? exception.cause : exception
+        super(exception, {:disconnect => cause.respond_to?(:getSQLState) && cause.getSQLState =~ /^08/}.merge(opts))
       end
-      
+
       # Java being java, you need to specify the type of each argument
       # for the prepared statement, and bind it individually.  This
       # guesses which JDBC method to use, and hopefully JRuby will convert
@@ -336,23 +395,27 @@ module Sequel
       def set_ps_arg(cps, arg, i)
         case arg
         when Integer
-          cps.setInt(i, arg)
+          cps.setLong(i, arg)
         when Sequel::SQL::Blob
           cps.setBytes(i, arg.to_java_bytes)
         when String
           cps.setString(i, arg)
-        when Date, Java::JavaSql::Date
-          cps.setDate(i, arg)
-        when DateTime, Java::JavaSql::Timestamp
-          cps.setTimestamp(i, arg)
-        when Time
-          cps.setTimestamp(i, java_sql_timestamp(arg))
         when Float
           cps.setDouble(i, arg)
         when TrueClass, FalseClass
           cps.setBoolean(i, arg)
         when nil
           cps.setNull(i, JavaSQL::Types::NULL)
+        when DateTime
+          cps.setTimestamp(i, java_sql_datetime(arg))
+        when Date
+          cps.setDate(i, java_sql_date(arg))
+        when Time
+          cps.setTimestamp(i, java_sql_timestamp(arg))
+        when Java::JavaSql::Timestamp
+          cps.setTimestamp(i, arg)
+        when Java::JavaSql::Date
+          cps.setDate(i, arg)
         else
           cps.setObject(i, arg)
         end
@@ -383,14 +446,30 @@ module Sequel
           pks << h[:column_name]
         end
         metadata(:getColumns, nil, schema, table, nil) do |h|
-          ts << [m.call(h[:column_name]), {:type=>schema_column_type(h[:type_name]), :db_type=>h[:type_name], :default=>(h[:column_def] == '' ? nil : h[:column_def]), :allow_null=>(h[:nullable] != 0), :primary_key=>pks.include?(h[:column_name]), :column_size=>h[:column_size]}]
+          s = {:type=>schema_column_type(h[:type_name]), :db_type=>h[:type_name], :default=>(h[:column_def] == '' ? nil : h[:column_def]), :allow_null=>(h[:nullable] != 0), :primary_key=>pks.include?(h[:column_name]), :column_size=>h[:column_size], :scale=>h[:decimal_digits]}
+          if s[:db_type] =~ DECIMAL_TYPE_RE && s[:scale] == 0
+            s[:type] = :integer
+          end
+          ts << [m.call(h[:column_name]), s]
         end
         ts
       end
       
-      # Create a statement object to execute transaction statements.
-      def transaction_statement_object(conn)
-        conn.createStatement
+      # Yield a new statement object, and ensure that it is closed before returning.
+      def statement(conn)
+        stmt = conn.createStatement
+        yield stmt
+      rescue NativeException, JavaSQL::SQLException => e
+        raise_error(e)
+      ensure
+        stmt.close if stmt
+      end
+
+      # This method determines whether or not to add
+      # Statement.RETURN_GENERATED_KEYS as an argument when inserting rows.
+      # Sub-adapters that require this should override this method.
+      def requires_return_generated_keys?
+        false
       end
     end
     
@@ -492,6 +571,8 @@ module Sequel
           BigDecimal.new(v.to_string)
         when Java::byte[]
           Sequel::SQL::Blob.new(String.from_java_bytes(v))
+        when Java::JavaSQL::Blob
+          convert_type(v.getBytes(1, v.length))
         else
           v
         end
@@ -511,19 +592,27 @@ module Sequel
         i = 0
         meta.getColumnCount.times{cols << [output_identifier(meta.getColumnLabel(i+=1)), i]}
         @columns = cols.map{|c| c.at(0)}
-        row = {}
-        blk = if @convert_types
-          lambda{|n, i| row[n] = convert_type(result.getObject(i))}
-        else
-          lambda{|n, i| row[n] = result.getObject(i)}
-        end
+        blk = result_set_object_getter
         # get rows
         while result.next
           row = {}
-          cols.each(&blk)
+          cols.each do |n, i|
+            row[n] = blk.call(result, n, i)
+          end
           yield row
         end
       end
+
+      # Proc that takes the ResultSet and an integer field number and returns
+      # the object for that field.  Respects the @convert_types setting to
+      # convert Java types to ruby types.
+      def result_set_object_getter
+        if @convert_types
+          lambda {|result, n, i| convert_type(result.getObject(i))}
+        else
+          lambda {|result, n, i| result.getObject(i)}
+        end
+      end
     end
   end
 end
