@@ -8,9 +8,12 @@ module Sequel
       SQL_BEGIN = "BEGIN TRANSACTION".freeze
       SQL_COMMIT = "COMMIT TRANSACTION".freeze
       SQL_ROLLBACK = "IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION".freeze
-      SQL_ROLLBACK_TO_SAVEPOINT = 'ROLLBACK TRANSACTION autopoint_%d'.freeze
+      SQL_ROLLBACK_TO_SAVEPOINT = 'IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION autopoint_%d'.freeze
       SQL_SAVEPOINT = 'SAVE TRANSACTION autopoint_%d'.freeze
-      TEMPORARY = "#".freeze
+      
+      # The types to check for 0 scale to transform :decimal types
+      # to :integer.
+      DECIMAL_TYPE_RE = /number|numeric|decimal/io
       
       # Microsoft SQL Server uses the :mssql type.
       def database_type
@@ -36,6 +39,11 @@ module Sequel
         true
       end
       
+      # MSSQL supports transaction isolation levels
+      def supports_transaction_isolation_levels?
+        true
+      end
+
       # Microsoft SQL Server supports using the INFORMATION_SCHEMA to get
       # information on tables.
       def tables(opts={})
@@ -59,14 +67,16 @@ module Sequel
         when :add_column
           "ALTER TABLE #{quote_schema_table(table)} ADD #{column_definition_sql(op)}"
         when :rename_column
-          "SP_RENAME #{literal("#{quote_schema_table(table)}.#{quote_identifier(op[:name])}")}, #{literal(op[:new_name].to_s)}, 'COLUMN'"
+          "sp_rename #{literal("#{quote_schema_table(table)}.#{quote_identifier(op[:name])}")}, #{literal(op[:new_name].to_s)}, 'COLUMN'"
         when :set_column_type
           "ALTER TABLE #{quote_schema_table(table)} ALTER COLUMN #{quote_identifier(op[:name])} #{type_literal(op)}"
         when :set_column_null
           sch = schema(table).find{|k,v| k.to_s == op[:name].to_s}.last
-          type = {:type=>sch[:db_type]}
-          type[:size] = sch[:max_chars] if sch[:max_chars]
-          "ALTER TABLE #{quote_schema_table(table)} ALTER COLUMN #{quote_identifier(op[:name])} #{type_literal(type)} #{'NOT ' unless op[:null]}NULL"
+          type = sch[:db_type]
+          if [:string, :decimal].include?(sch[:type]) and size = (sch[:max_chars] || sch[:column_size])
+            type += "(#{size}#{", #{sch[:scale]}" if sch[:scale] && sch[:scale].to_i > 0})"
+          end
+          "ALTER TABLE #{quote_schema_table(table)} ALTER COLUMN #{quote_identifier(op[:name])} #{type_literal(:type=>type)} #{'NOT ' unless op[:null]}NULL"
         when :set_column_default
           "ALTER TABLE #{quote_schema_table(table)} ADD CONSTRAINT #{quote_identifier("sequel_#{table}_#{op[:name]}_def")} DEFAULT #{literal(op[:default])} FOR #{quote_identifier(op[:name])}"
         else
@@ -86,7 +96,7 @@ module Sequel
       
       # Commit the active transaction on the connection, does not commit/release
       # savepoints.
-      def commit_transaction(conn)
+      def commit_transaction(conn, opts={})
         log_connection_execute(conn, commit_transaction_sql) unless Thread.current[:sequel_transaction_depth] > 1
       end
 
@@ -94,6 +104,13 @@ module Sequel
       def commit_transaction_sql
         SQL_COMMIT
       end
+        
+      # MSSQL uses the name of the table to decide the difference between
+      # a regular and temporary table, with temporary table names starting with
+      # a #.
+      def create_table_sql(name, generator, options)
+        "CREATE TABLE #{quote_schema_table(options[:temp] ? "##{name}" : name)} (#{column_list_sql(generator)})"
+      end
       
       # The SQL to drop an index for the table.
       def drop_index_sql(table, op)
@@ -107,6 +124,11 @@ module Sequel
         ds
       end
       
+      # Use sp_rename to rename the table
+      def rename_table_sql(name, new_name)
+        "sp_rename #{literal(quote_schema_table(name))}, #{quote_identifier(schema_and_table(new_name).pop)}"
+      end
+      
       # SQL to rollback to a savepoint
       def rollback_savepoint_sql(depth)
         SQL_ROLLBACK_TO_SAVEPOINT % depth
@@ -125,7 +147,7 @@ module Sequel
         ds = metadata_dataset.from(:information_schema__tables___t).
          join(:information_schema__columns___c, :table_catalog=>:table_catalog,
               :table_schema => :table_schema, :table_name => :table_name).
-         select(:column_name___column, :data_type___db_type, :character_maximum_length___max_chars, :column_default___default, :is_nullable___allow_null).
+         select(:column_name___column, :data_type___db_type, :character_maximum_length___max_chars, :column_default___default, :is_nullable___allow_null, :numeric_precision___column_size, :numeric_scale___scale).
          filter(:c__table_name=>m2.call(table_name.to_s))
         if schema = opts[:schema] || default_schema
           ds.filter!(:c__table_schema=>schema)
@@ -133,16 +155,15 @@ module Sequel
         ds.map do |row|
           row[:allow_null] = row[:allow_null] == 'YES' ? true : false
           row[:default] = nil if blank_object?(row[:default])
-          row[:type] = schema_column_type(row[:db_type])
+          row[:type] = if row[:db_type] =~ DECIMAL_TYPE_RE && row[:scale] == 0
+            :integer
+          else
+            schema_column_type(row[:db_type])
+          end
           [m.call(row.delete(:column)), row]
         end
       end
       
-      # SQL fragment for marking a table as temporary
-      def temporary_table_sql
-        TEMPORARY
-      end
-      
       # MSSQL has both datetime and timestamp classes, most people are going
       # to want datetime
       def type_literal_generic_datetime(column)
@@ -160,9 +181,9 @@ module Sequel
         :bit
       end
       
-      # MSSQL uses image type for blobs
+      # MSSQL uses varbinary(max) type for blobs
       def type_literal_generic_file(column)
-        :image
+        :'varbinary(max)'
       end
 
       # support for clustered index type
@@ -183,8 +204,10 @@ module Sequel
       COMMA_SEPARATOR = ', '.freeze
       DELETE_CLAUSE_METHODS = Dataset.clause_methods(:delete, %w'with from output from2 where')
       INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'with into columns output values')
-      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'with limit distinct columns from table_options join where group order having compounds')
+      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'with distinct limit columns into from lock join where group having order compounds')
       UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'with table set output from where')
+      NOLOCK = ' WITH (NOLOCK)'.freeze
+      UPDLOCK = ' WITH (UPDLOCK)'.freeze
       WILDCARD = LiteralString.new('*').freeze
       CONSTANT_MAP = {:CURRENT_DATE=>'CAST(CURRENT_TIMESTAMP AS DATE)'.freeze, :CURRENT_TIME=>'CAST(CURRENT_TIMESTAMP AS TIME)'.freeze}
 
@@ -197,6 +220,10 @@ module Sequel
           super(:LIKE, args)
         when :"NOT ILIKE"
           super(:"NOT LIKE", args)
+        when :<<
+          "(#{literal(args[0])} * POWER(2, #{literal(args[1])}))"
+        when :>>
+          "(#{literal(args[0])} / POWER(2, #{literal(args[1])}))"
         else
           super(op, args)
         end
@@ -230,17 +257,23 @@ module Sequel
 
       # Use the OUTPUT clause to get the value of all columns for the newly inserted record.
       def insert_select(*values)
+        return unless supports_output_clause?
         naked.clone(default_server_opts(:sql=>output(nil, [:inserted.*]).insert_sql(*values))).single_record unless opts[:disable_insert_output]
       end
 
+      # Specify a table for a SELECT ... INTO query.
+      def into(table)
+        clone(:into => table)
+      end
+
       # MSSQL uses a UNION ALL statement to insert multiple values at once.
       def multi_insert_sql(columns, values)
         [insert_sql(columns, LiteralString.new(values.map {|r| "SELECT #{expression_list(r)}" }.join(" UNION ALL ")))]
       end
 
-      # Allows you to do .nolock on a query
+      # Allows you to do a dirty read of uncommitted data using WITH (NOLOCK).
       def nolock
-        clone(:table_options => "(NOLOCK)")
+        lock_style(:dirty)
       end
 
       # Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
@@ -256,6 +289,7 @@ module Sequel
       #   dataset.output(:output_table, [:deleted__id, :deleted__name])
       #   dataset.output(:output_table, :id => :inserted__id, :name => :inserted__name)
       def output(into, values)
+        raise(Error, "SQL Server versions 2000 and earlier do not support the OUTPUT clause") unless supports_output_clause?
         output = {}
         case values
           when Hash
@@ -283,7 +317,7 @@ module Sequel
       # 
       # The implementation is ugly, cloning the current dataset and modifying
       # the clone to add a ROW_NUMBER window function (and some other things),
-      # then using the modified clone in a CTE which is selected from.
+      # then using the modified clone in a subselect which is selected from.
       #
       # If offset is used, an order must be provided, because the use of ROW_NUMBER
       # requires an order.
@@ -291,16 +325,13 @@ module Sequel
         return super unless o = @opts[:offset]
         raise(Error, 'MSSQL requires an order be provided if using an offset') unless order = @opts[:order]
         dsa1 = dataset_alias(1)
-        dsa2 = dataset_alias(2)
         rn = row_number_column
-        unlimited.
+        subselect_sql(unlimited.
           unordered.
-          from_self(:alias=>dsa2).
-          select{[WILDCARD, ROW_NUMBER(:over, :order=>order){}.as(rn)]}.
+          select_append{ROW_NUMBER(:over, :order=>order){}.as(rn)}.
           from_self(:alias=>dsa1).
           limit(@opts[:limit]).
-          where(rn > o).
-          select_sql
+          where(SQL::Identifier.new(rn) > o))
       end
 
       # The version of the database server.
@@ -308,9 +339,9 @@ module Sequel
         db.server_version(@opts[:server])
       end
 
-      # Microsoft SQL Server does not support INTERSECT or EXCEPT
+      # MSSQL 2005+ supports INTERSECT and EXCEPT
       def supports_intersect_except?
-        false
+        is_2005_or_later?
       end
       
       # MSSQL does not support IS TRUE
@@ -323,21 +354,35 @@ module Sequel
         false
       end
 
+      # MSSQL 2005+ supports modifying joined datasets
+      def supports_modifying_joins?
+        is_2005_or_later?
+      end
+
       # MSSQL does not support multiple columns for the IN/NOT IN operators
       def supports_multiple_column_in?
         false
       end
+      
+      # MSSQL 2005+ supports the output clause.
+      def supports_output_clause?
+        is_2005_or_later?
+      end
 
       # MSSQL 2005+ supports window functions
       def supports_window_functions?
         true
       end
+      
+      protected
+      # MSSQL does not allow ordering in sub-clauses unless 'top' (limit) is specified
+      def aggregate_dataset
+        (options_overlap(Sequel::Dataset::COUNT_FROM_SELF_OPTS) && !options_overlap([:limit])) ? unordered.from_self : super
+      end
 
       private
-
-      # MSSQL can modify joined datasets
-      def check_modification_allowed!
-        raise(InvalidOperation, "Grouped datasets cannot be modified") if opts[:group]
+      def is_2005_or_later?
+        server_version >= 9000000
       end
 
       # MSSQL supports the OUTPUT clause for DELETE statements.
@@ -346,6 +391,20 @@ module Sequel
         DELETE_CLAUSE_METHODS
       end
 
+      # Only include the primary table in the main delete clause
+      def delete_from_sql(sql)
+        sql << " FROM #{source_list(@opts[:from][0..0])}"
+      end
+
+      # MSSQL supports FROM clauses in DELETE and UPDATE statements.
+      def delete_from2_sql(sql)
+        if joined_dataset?
+          select_from_sql(sql)
+          select_join_sql(sql)
+        end
+      end
+      alias update_from_sql delete_from2_sql
+      
       # Handle the with clause for delete, insert, and update statements
       # to be the same as the insert statement.
       def delete_with_sql(sql)
@@ -361,16 +420,6 @@ module Sequel
         sprintf(".%03d", usec/1000)
       end
 
-      # MSSQL supports FROM clauses in DELETE and UPDATE statements.
-      def from_sql(sql)
-        if (opts[:from].is_a?(Array) && opts[:from].size > 1) || opts[:join]
-          select_from_sql(sql)
-          select_join_sql(sql)
-        end
-      end
-      alias delete_from2_sql from_sql
-      alias update_from_sql from_sql
-      
       # MSSQL supports the OUTPUT clause for INSERT statements.
       # It also allows prepending a WITH clause.
       def insert_clause_methods
@@ -384,9 +433,9 @@ module Sequel
         blob
       end
       
-      # Use unicode string syntax for all strings
+      # Use unicode string syntax for all strings. Don't double backslashes.
       def literal_string(v)
-        "N#{super}"
+        "N'#{v.gsub(/'/, "''")}'"
       end
       
       # Use 0 for false on MSSQL
@@ -409,18 +458,35 @@ module Sequel
         SELECT_CLAUSE_METHODS
       end
 
-      # MSSQL uses TOP for limit
-      def select_limit_sql(sql)
-        sql << " TOP #{@opts[:limit]}" if @opts[:limit]
+      def select_into_sql(sql)
+        sql << " INTO #{table_ref(@opts[:into])}" if @opts[:into]
       end
 
-      # MSSQL uses the WITH statement to lock tables
-      def select_table_options_sql(sql)
-        sql << " WITH #{@opts[:table_options]}" if @opts[:table_options]
+      # MSSQL uses TOP N for limit.  For MSSQL 2005+ TOP (N) is used
+      # to allow the limit to be a bound variable.
+      def select_limit_sql(sql)
+        if l = @opts[:limit]
+          l = literal(l)
+          l = "(#{l})" if server_version >= 9000000
+          sql << " TOP #{l}"
+        end
+      end
+
+      # Support different types of locking styles
+      def select_lock_sql(sql)
+        case @opts[:lock]
+        when :update
+          sql << UPDLOCK
+        when :dirty
+          sql << NOLOCK
+        else
+          super
+        end
       end
 
       # SQL fragment for MSSQL's OUTPUT clause.
       def output_sql(sql)
+        return unless supports_output_clause?
         return unless output = @opts[:output]
         sql << " OUTPUT #{column_list(output[:select_list])}"
         if into = output[:into]
@@ -441,6 +507,11 @@ module Sequel
       def update_clause_methods
         UPDATE_CLAUSE_METHODS
       end
+
+      # Only include the primary table in the main update clause
+      def update_table_sql(sql)
+        sql << " #{source_list(@opts[:from][0..0])}"
+      end
     end
   end
 end
