@@ -117,10 +117,8 @@ module Sequel
 
         # Serialize all deserialized values
         def before_save
+          deserialized_values.each{|k,v| @values[k] = serialize_value(k, v)}
           super
-          deserialized_values.each do |k,v|
-            @values[k] = serialize_value(k, v)
-          end
         end
         
         # Empty the deserialized values when refreshing.
@@ -155,7 +153,7 @@ module Sequel
           when :yaml
             v.to_yaml
           when :json
-            JSON.generate v
+            v.to_json
           else
             raise Error, "Bad serialization format (#{model.serialization_map[column].inspect}) for column #{column.inspect}"
           end
