@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), 'spec_helper')
+require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 
 context "DB#create_table" do
   before do
@@ -23,6 +23,12 @@ context "DB#create_table" do
     proc{@db.create_table(:cats.as(:c)) {}}.should raise_error(Sequel::Error)
   end
 
+  specify "should remove cached schema entry" do
+    @db.instance_variable_set(:@schemas, {'cats'=>[]})
+    @db.create_table(:cats){Integer :a}
+    @db.instance_variable_get(:@schemas).should be_empty
+  end
+  
   specify "should accept multiple columns" do
     @db.create_table(:cats) do
       column :id, :integer
@@ -268,6 +274,13 @@ context "DB#create_table" do
     @db.sqls.clear
   end
   
+  specify "should accept foreign keys with deferrable option" do
+    @db.create_table(:cats) do
+      foreign_key :project_id, :projects, :deferrable=>true
+    end
+    @db.sqls.should == ["CREATE TABLE cats (project_id integer REFERENCES projects DEFERRABLE INITIALLY DEFERRED)"]
+  end
+
   specify "should accept inline index definition" do
     @db.create_table(:cats) do
       integer :id, :index => true
@@ -299,6 +312,15 @@ context "DB#create_table" do
     @db.sqls.should == ["CREATE TABLE cats (name text, UNIQUE (name))"]
   end
 
+  specify "should not raise on index error for unsupported index definitions if ignore_index_errors is used" do
+    proc {
+      @db.create_table(:cats, :ignore_index_errors=>true) do
+        text :name
+        full_text_index :name
+      end
+    }.should_not raise_error
+  end
+
   specify "should raise on full-text index definitions" do
     proc {
       @db.create_table(:cats) do
@@ -812,8 +834,12 @@ context "Schema Parser" do
     @db.schema(:float).first.last[:type].should == :float
     @db.schema(:double).first.last[:type].should == :float
     @db.schema(:"double precision").first.last[:type].should == :float
+    @db.schema(:number).first.last[:type].should == :decimal
     @db.schema(:numeric).first.last[:type].should == :decimal
     @db.schema(:decimal).first.last[:type].should == :decimal
+    @db.schema(:"number(10,0)").first.last[:type].should == :integer
+    @db.schema(:"numeric(10, 10)").first.last[:type].should == :decimal
+    @db.schema(:"decimal(10,1)").first.last[:type].should == :decimal
     @db.schema(:money).first.last[:type].should == :decimal
     @db.schema(:bytea).first.last[:type].should == :blob
     @db.schema(:blob).first.last[:type].should == :blob
