@@ -1,6 +1,7 @@
 require 'rubygems'
+
 unless Object.const_defined?('Sequel')
-  $:.unshift(File.join(File.dirname(__FILE__), "../../lib/"))
+  $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
   require 'sequel/core'
 end
 
@@ -24,6 +25,7 @@ class MockDataset < Sequel::Dataset
 end
 
 class MockDatabase < Sequel::Database
+  set_adapter_scheme :mock
   @@quote_identifiers = false
   self.identifier_input_method = nil
   self.identifier_output_method = nil
