@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), "spec_helper")
+require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe "Model#after_initialize" do
   specify "should be called after initialization" do
@@ -89,6 +89,14 @@ describe "Model#before_update && Model#after_update" do
     MODEL_DB.sqls.should == []
   end
 
+  specify "#save should cancel the save and raise an error if before_update returns false and raise_on_failure option is true" do
+    @c.send(:define_method, :before_update){false}
+    @c.raise_on_save_failure = false
+    proc{@c.load(:id => 2233).save(:raise_on_failure => true)}.should_not raise_error(Sequel::ValidationFailed)
+    proc{@c.load(:id => 2233).save(:raise_on_failure => true)}.should raise_error(Sequel::BeforeHookFailed)
+    MODEL_DB.sqls.should == []
+  end
+
   specify "#save should cancel the save and return nil if before_update returns false and raise_on_save_failure is false" do
     @c.send(:define_method, :before_update){false}
     @c.raise_on_save_failure = false
@@ -137,6 +145,14 @@ describe "Model#before_save && Model#after_save" do
     MODEL_DB.sqls.should == []
   end
 
+  specify "#save should cancel the save and raise an error if before_save returns false and raise_on_failure option is true" do
+    @c.send(:define_method, :before_save){false}
+    @c.raise_on_save_failure = false
+    proc{@c.load(:id => 2233).save(:raise_on_failure => true)}.should_not raise_error(Sequel::ValidationFailed)
+    proc{@c.load(:id => 2233).save(:raise_on_failure => true)}.should raise_error(Sequel::BeforeHookFailed)
+    MODEL_DB.sqls.should == []
+  end
+
   specify "#save should cancel the save and return nil if before_save returns false and raise_on_save_failure is false" do
     @c.send(:define_method, :before_save){false}
     @c.raise_on_save_failure = false
@@ -176,6 +192,13 @@ describe "Model#before_destroy && Model#after_destroy" do
     MODEL_DB.sqls.should == []
   end
 
+  specify "#destroy should cancel the destroy and raise an error if before_destroy returns false and raise_on_failure option is true" do
+    @c.send(:define_method, :before_destroy){false}
+    @c.raise_on_save_failure = false
+    proc{@c.load(:id => 2233).destroy(:raise_on_failure => true)}.should raise_error(Sequel::BeforeHookFailed)
+    MODEL_DB.sqls.should == []
+  end
+
   specify "#destroy should cancel the destroy and return nil if before_destroy returns false and raise_on_save_failure is false" do
     @c.send(:define_method, :before_destroy){false}
     @c.raise_on_save_failure = false
@@ -231,6 +254,14 @@ describe "Model#before_validation && Model#after_validation" do
     MODEL_DB.sqls.should == []
   end
 
+  specify "#save should cancel the save and raise an error if before_validation returns false and raise_on_failure option is true" do
+    @c.send(:define_method, :before_validation){false}
+    @c.raise_on_save_failure = false
+    proc{@c.load(:id => 2233).save(:raise_on_failure => true)}.should_not raise_error(Sequel::ValidationFailed)
+    proc{@c.load(:id => 2233).save(:raise_on_failure => true)}.should raise_error(Sequel::BeforeHookFailed)
+    MODEL_DB.sqls.should == []
+  end
+  
   specify "#save should cancel the save and return nil if before_validation returns false and raise_on_save_failure is false" do
     @c.send(:define_method, :before_validation){false}
     @c.raise_on_save_failure = false
