@@ -1,4 +1,4 @@
-require File.join(File.dirname(__FILE__), "spec_helper")
+require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe "Model#save server use" do
   
@@ -63,7 +63,7 @@ describe "Model#save" do
     MODEL_DB.sqls.should == ["INSERT INTO items (y) VALUES (2)"]
   end
 
-  it "should use value returned by insert as the primary key" do
+  it "should use value returned by insert as the primary key and refresh the object" do
     @c.dataset.meta_def(:insert){|h| super(h); 13}
     o = @c.new(:x => 11)
     o.save
@@ -71,6 +71,13 @@ describe "Model#save" do
       "SELECT * FROM items WHERE (id = 13) LIMIT 1"]
   end
 
+  it "should allow you to skip refreshing by overridding _save_refresh" do
+    @c.dataset.meta_def(:insert){|h| super(h); 13}
+    @c.send(:define_method, :_save_refresh){}
+    @c.create(:x => 11)
+    MODEL_DB.sqls.should == ["INSERT INTO items (x) VALUES (11)"]
+  end
+
   it "should work correctly for inserting a record without a primary key" do
     @c.dataset.meta_def(:insert){|h| super(h); 13}
     @c.no_primary_key
@@ -97,6 +104,22 @@ describe "Model#save" do
     MODEL_DB.sqls.should == ["UPDATE items SET x = 1 WHERE (id = 3)"]
   end
   
+  it "should raise a NoExistingObject exception if the dataset update call doesn't return 1, unless require_modification is false" do
+    o = @c.load(:id => 3, :x => 1)
+    o.this.meta_def(:update){|*a| 0}
+    proc{o.save}.should raise_error(Sequel::NoExistingObject)
+    o.this.meta_def(:update){|*a| 2}
+    proc{o.save}.should raise_error(Sequel::NoExistingObject)
+    o.this.meta_def(:update){|*a| 1}
+    proc{o.save}.should_not raise_error
+    
+    o.require_modification = false
+    o.this.meta_def(:update){|*a| 0}
+    proc{o.save}.should_not raise_error
+    o.this.meta_def(:update){|*a| 2}
+    proc{o.save}.should_not raise_error
+  end
+  
   it "should update only the given columns if given" do
     o = @c.load(:id => 3, :x => 1, :y => nil)
     o.save(:y)
@@ -158,7 +181,7 @@ describe "Model#save" do
     res.should == [nil, nil]
   end
   
-  it "should use Model's save_in_transaction setting by default" do
+  it "should use Model's use_transactions setting by default" do
     @c.use_transactions = true
     @c.load(:id => 3, :x => 1, :y => nil).save(:y)
     MODEL_DB.sqls.should == ["BEGIN", "UPDATE items SET y = NULL WHERE (id = 3)", "COMMIT"]
@@ -169,7 +192,7 @@ describe "Model#save" do
     MODEL_DB.reset
   end
 
-  it "should inherit Model's save_in_transaction setting" do
+  it "should inherit Model's use_transactions setting" do
     @c.use_transactions = true
     Class.new(@c).load(:id => 3, :x => 1, :y => nil).save(:y)
     MODEL_DB.sqls.should == ["BEGIN", "UPDATE items SET y = NULL WHERE (id = 3)", "COMMIT"]
@@ -180,7 +203,7 @@ describe "Model#save" do
     MODEL_DB.reset
   end
 
-  it "should use object's save_in_transaction setting" do
+  it "should use object's use_transactions setting" do
     o = @c.load(:id => 3, :x => 1, :y => nil)
     o.use_transactions = false
     @c.use_transactions = true
@@ -207,6 +230,68 @@ describe "Model#save" do
     MODEL_DB.sqls.should == ["BEGIN", "UPDATE items SET y = NULL WHERE (id = 3)", "COMMIT"]
     MODEL_DB.reset
   end
+
+  it "should rollback if before_save returns false and raise_on_save_failure = true" do
+    o = @c.load(:id => 3, :x => 1, :y => nil)
+    o.use_transactions = true
+    o.raise_on_save_failure = true
+    def o.before_save
+      false
+    end
+    proc { o.save(:y) }.should raise_error(Sequel::BeforeHookFailed)
+    MODEL_DB.sqls.should == ["BEGIN", "ROLLBACK"]
+    MODEL_DB.reset
+  end
+
+  it "should rollback if before_save returns false and :raise_on_failure option is true" do
+    o = @c.load(:id => 3, :x => 1, :y => nil)
+    o.use_transactions = true
+    o.raise_on_save_failure = false
+    def o.before_save
+      false
+    end
+    proc { o.save(:y, :raise_on_failure => true) }.should raise_error(Sequel::BeforeHookFailed)
+    MODEL_DB.sqls.should == ["BEGIN", "ROLLBACK"]
+    MODEL_DB.reset
+  end
+
+  it "should not rollback outer transactions if before_save returns false and raise_on_save_failure = false" do
+    o = @c.load(:id => 3, :x => 1, :y => nil)
+    o.use_transactions = true
+    o.raise_on_save_failure = false
+    def o.before_save
+      false
+    end
+    MODEL_DB.transaction do
+      o.save(:y).should == nil
+      MODEL_DB.run "BLAH"
+    end
+    MODEL_DB.sqls.should == ["BEGIN", "BLAH", "COMMIT"]
+    MODEL_DB.reset
+  end
+
+  it "should rollback if before_save returns false and raise_on_save_failure = false" do
+    o = @c.load(:id => 3, :x => 1, :y => nil)
+    o.use_transactions = true
+    o.raise_on_save_failure = false
+    def o.before_save
+      false
+    end
+    o.save(:y).should == nil
+    MODEL_DB.sqls.should == ["BEGIN", "ROLLBACK"]
+    MODEL_DB.reset
+  end
+
+  it "should not rollback if before_save throws Rollback and use_transactions = false" do
+    o = @c.load(:id => 3, :x => 1, :y => nil)
+    o.use_transactions = false
+    def o.before_save
+      raise Sequel::Rollback
+    end
+    proc { o.save(:y) }.should raise_error(Sequel::Rollback)
+    MODEL_DB.sqls.should == []
+    MODEL_DB.reset
+  end
 end
 
 describe "Model#marshallable" do
@@ -313,7 +398,7 @@ describe "Model#save_changes" do
 
   it "should take options passed to save" do
     o = @c.new(:x => 1)
-    def o.valid?; false; end
+    def o.before_validation; false; end
     proc{o.save_changes}.should raise_error(Sequel::Error)
     MODEL_DB.sqls.should == []
     o.save_changes(:validate=>false)
@@ -630,6 +715,30 @@ describe Sequel::Model, "#set" do
     returned_value.should == @o1
     MODEL_DB.sqls.should == []
   end
+
+  it "#set should correctly handle cases where an instance method is added to the class" do
+    @o1.set(:x => 1)
+    @o1.values.should == {:x => 1}
+
+    @c.class_eval do
+      def z=(v)
+        self[:z] = v
+      end
+    end
+    @o1.set(:x => 2, :z => 3)
+    @o1.values.should == {:x => 2, :z=>3}
+  end
+
+  it "#set should correctly handle cases where a singleton method is added to the object" do
+    @o1.set(:x => 1)
+    @o1.values.should == {:x => 1}
+
+    def @o1.z=(v)
+      self[:z] = v
+    end
+    @o1.set(:x => 2, :z => 3)
+    @o1.values.should == {:x => 2, :z=>3}
+  end
 end
 
 describe Sequel::Model, "#update" do
@@ -671,6 +780,54 @@ describe Sequel::Model, "#update" do
   end
 end
 
+describe Sequel::Model, "#set_fields" do
+  before do
+    @c = Class.new(Sequel::Model(:items))
+    @c.class_eval do
+      set_primary_key :id
+      columns :x, :y, :z, :id
+    end
+    @c.strict_param_setting = true 
+    @o1 = @c.new
+    MODEL_DB.reset
+  end
+
+  it "should set only the given fields" do
+    @o1.set_fields({:x => 1, :y => 2, :z=>3, :id=>4}, [:x, :y])
+    @o1.values.should == {:x => 1, :y => 2}
+    @o1.set_fields({:x => 9, :y => 8, :z=>6, :id=>7}, [:x, :y, :id])
+    @o1.values.should == {:x => 9, :y => 8, :id=>7}
+    MODEL_DB.sqls.should == []
+  end
+end
+
+describe Sequel::Model, "#update_fields" do
+  before do
+    @c = Class.new(Sequel::Model(:items))
+    @c.class_eval do
+      set_primary_key :id
+      columns :x, :y, :z, :id
+      def _refresh(ds); end
+    end
+    @c.strict_param_setting = true 
+    @o1 = @c.load(:id=>1)
+    MODEL_DB.reset
+  end
+
+  it "should set only the given fields, and then save the changes to the record" do
+    @o1.update_fields({:x => 1, :y => 2, :z=>3, :id=>4}, [:x, :y])
+    @o1.values.should == {:x => 1, :y => 2, :id=>1}
+    MODEL_DB.sqls.first.should =~ /UPDATE items SET [xy] = [12], [xy] = [12] WHERE \(id = 1\)/
+    MODEL_DB.sqls.length.should == 1
+    MODEL_DB.reset
+
+    @o1.update_fields({:x => 1, :y => 5, :z=>6, :id=>7}, [:x, :y])
+    @o1.values.should == {:x => 1, :y => 5, :id=>1}
+    MODEL_DB.sqls.should == ["UPDATE items SET y = 5 WHERE (id = 1)"]
+    MODEL_DB.reset
+  end
+end
+
 describe Sequel::Model, "#(set|update)_(all|except|only)" do
   before do
     MODEL_DB.reset
@@ -686,7 +843,25 @@ describe Sequel::Model, "#(set|update)_(all|except|only)" do
     @o1 = @c.new
   end
 
-  it "#set_all should set all attributes" do
+  it "should raise errors if not all hash fields can be set and strict_param_setting is true" do
+    @c.strict_param_setting = true
+
+    proc{@c.new.set_all(:x => 1, :y => 2, :z=>3, :id=>4)}.should raise_error(Sequel::Error)
+    (o = @c.new).set_all(:x => 1, :y => 2, :z=>3)
+    o.values.should == {:x => 1, :y => 2, :z=>3}
+
+    proc{@c.new.set_only({:x => 1, :y => 2, :z=>3, :id=>4}, :x, :y)}.should raise_error(Sequel::Error)
+    proc{@c.new.set_only({:x => 1, :y => 2, :z=>3}, :x, :y)}.should raise_error(Sequel::Error)
+    (o = @c.new).set_only({:x => 1, :y => 2}, :x, :y)
+    o.values.should == {:x => 1, :y => 2}
+
+    proc{@c.new.set_except({:x => 1, :y => 2, :z=>3, :id=>4}, :x, :y)}.should raise_error(Sequel::Error)
+    proc{@c.new.set_except({:x => 1, :y => 2, :z=>3}, :x, :y)}.should raise_error(Sequel::Error)
+    (o = @c.new).set_except({:z => 3}, :x, :y)
+    o.values.should == {:z=>3}
+  end
+
+  it "#set_all should set all attributes except the primary key" do
     @o1.set_all(:x => 1, :y => 2, :z=>3, :id=>4)
     @o1.values.should == {:x => 1, :y => 2, :z=>3}
   end
@@ -696,9 +871,11 @@ describe Sequel::Model, "#(set|update)_(all|except|only)" do
     @o1.values.should == {:x => 1, :y => 2}
     @o1.set_only({:x => 4, :y => 5, :z=>6, :id=>7}, :x, :y)
     @o1.values.should == {:x => 4, :y => 5}
+    @o1.set_only({:x => 9, :y => 8, :z=>6, :id=>7}, :x, :y, :id)
+    @o1.values.should == {:x => 9, :y => 8, :id=>7}
   end
 
-  it "#set_except should not set given attributes" do
+  it "#set_except should not set given attributes or the primary key" do
     @o1.set_except({:x => 1, :y => 2, :z=>3, :id=>4}, [:y, :z])
     @o1.values.should == {:x => 1}
     @o1.set_except({:x => 4, :y => 2, :z=>3, :id=>4}, :y, :z)
@@ -738,7 +915,7 @@ describe Sequel::Model, "#destroy" do
     MODEL_DB.reset
     @model = Class.new(Sequel::Model(:items))
     @model.columns :id
-    @model.dataset.meta_def(:delete) {MODEL_DB.execute delete_sql}
+    @model.dataset.meta_def(:delete){MODEL_DB.execute delete_sql;1}
     
     @instance = @model.load(:id => 1234)
   end
@@ -747,6 +924,21 @@ describe Sequel::Model, "#destroy" do
     @model.send(:define_method, :after_destroy){3}
     @instance.destroy.should == @instance
   end
+  
+  it "should raise a NoExistingObject exception if the dataset delete call doesn't return 1" do
+    @instance.this.meta_def(:delete){|*a| 0}
+    proc{@instance.delete}.should raise_error(Sequel::NoExistingObject)
+    @instance.this.meta_def(:delete){|*a| 2}
+    proc{@instance.delete}.should raise_error(Sequel::NoExistingObject)
+    @instance.this.meta_def(:delete){|*a| 1}
+    proc{@instance.delete}.should_not raise_error
+    
+    @instance.require_modification = false
+    @instance.this.meta_def(:delete){|*a| 0}
+    proc{@instance.delete}.should_not raise_error
+    @instance.this.meta_def(:delete){|*a| 2}
+    proc{@instance.delete}.should_not raise_error
+  end
 
   it "should run within a transaction if use_transactions is true" do
     @instance.use_transactions = true
@@ -760,6 +952,18 @@ describe Sequel::Model, "#destroy" do
     @instance.destroy
   end
 
+  it "should run within a transaction if :transaction option is true" do
+    @instance.use_transactions = false
+    @model.db.should_receive(:transaction)
+    @instance.destroy(:transaction => true)
+  end
+
+  it "should not run within a transaction if :transaction option is false" do
+    @instance.use_transactions = true
+    @model.db.should_not_receive(:transaction)
+    @instance.destroy(:transaction => false)
+  end
+
   it "should run before_destroy and after_destroy hooks" do
     @model.send(:define_method, :before_destroy){MODEL_DB.execute('before blah')}
     @model.send(:define_method, :after_destroy){MODEL_DB.execute('after blah')}
@@ -776,17 +980,22 @@ end
 describe Sequel::Model, "#exists?" do
   before(:each) do
     @model = Class.new(Sequel::Model(:items))
-    @m = @model.new
+    @ds = @model.dataset
+    def @ds.fetch_rows(sql)
+      db.execute(sql)
+      yield(:x=>1) if sql =~ /id = 1/
+    end
+    MODEL_DB.reset
   end
 
-  it "should returns true when #this.count > 0" do
-    @m.this.meta_def(:count) {1}
-    @m.exists?.should be_true
+  it "should do a query to check if the record exists" do
+    @model.load(:id=>1).exists?.should be_true
+    MODEL_DB.sqls.should == ['SELECT 1 FROM items WHERE (id = 1) LIMIT 1']
   end
 
   it "should return false when #this.count == 0" do
-    @m.this.meta_def(:count) {0}
-    @m.exists?.should be_false
+    @model.load(:id=>2).exists?.should be_false
+    MODEL_DB.sqls.should == ['SELECT 1 FROM items WHERE (id = 2) LIMIT 1']
   end
 end
 
@@ -1294,7 +1503,6 @@ describe Sequel::Model, "typecasting" do
   specify "should raise an error if invalid data is used in a time field" do
     @c.instance_variable_set(:@db_schema, {:x=>{:type=>:time}})
     proc{@c.new.x = '0000'}.should raise_error
-    proc{@c.new.x = 'a'}.should_not raise_error # Valid Time
     proc{@c.new.x = Date.parse('2008-10-21')}.should raise_error(Sequel::InvalidValue)
     proc{@c.new.x = DateTime.parse('2008-10-21')}.should raise_error(Sequel::InvalidValue)
   end
@@ -1351,7 +1559,6 @@ describe Sequel::Model, "typecasting" do
   specify "should raise an error if invalid data is used in a datetime field" do
     @c.instance_variable_set(:@db_schema, {:x=>{:type=>:datetime}})
     proc{@c.new.x = '0000'}.should raise_error(Sequel::InvalidValue)
-    proc{@c.new.x = 'a'}.should_not raise_error # Valid Time
     Sequel.datetime_class = DateTime
     proc{@c.new.x = '0000'}.should raise_error(Sequel::InvalidValue)
     proc{@c.new.x = 'a'}.should raise_error(Sequel::InvalidValue)
@@ -1371,3 +1578,37 @@ describe Sequel::Model, "typecasting" do
     model.x.should == 'a'
   end
 end
+
+describe "Model#lock!" do
+  before do
+    @c = Class.new(Sequel::Model(:items)) do
+      columns :id
+    end
+    ds = @c.dataset
+    def ds.fetch_rows(sql)
+      db.execute(sql)
+      yield({:id=>1})
+    end
+    MODEL_DB.reset
+  end
+  
+  it "should do nothing if the record is a new record" do
+    o = @c.new
+    called = false
+    o.meta_def(:_refresh){|x| called = true; super(x)}
+    x = o.lock!
+    x.should == o
+    called.should == false
+    MODEL_DB.sqls.should == []
+  end
+    
+  it "should refresh the record using for_update if it is not a new record" do
+    o = @c.load(:id => 1)
+    called = false
+    o.meta_def(:_refresh){|x| called = true; super(x)}
+    x = o.lock!
+    x.should == o
+    called.should == true
+    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (id = 1) LIMIT 1 FOR UPDATE"]
+  end
+end
